                       /**
 * Secure channel for Java Card
 * 
 * Edited by Petr Skyva
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * 
 * 
 * Copyright (C) Pim Vullers, Radboud University Nijmegen, February 2011.
 */
package Applet;

import javacard.framework.*;
import javacard.security.*;
import javacardx.crypto.*;
import javax.smartcardio.ResponseAPDU;
import static jpass.smartcard.TerminalManager.byteToHex;


public class SecureChannel {
    private static final short KEY_SIZE = 16;
    private static final byte[] EMPTY_KEY = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
    
    private Cipher cipher;
    private Cipher decipher;
    
    private DESKey keyMAC;
    private DESKey keyENC;
    
    public SecureChannel() {
        cipher = Cipher.getInstance(
                Cipher.ALG_DES_CBC_ISO9797_M2, false);
        decipher = Cipher.getInstance(
                Cipher.ALG_DES_CBC_ISO9797_M2, false);
        
        keyMAC = (DESKey) KeyBuilder.buildKey(
                KeyBuilder.TYPE_DES_TRANSIENT_DESELECT, 
                KeyBuilder.LENGTH_DES3_2KEY, false);
        keyENC = (DESKey) KeyBuilder.buildKey(
                KeyBuilder.TYPE_DES_TRANSIENT_DESELECT, 
                KeyBuilder.LENGTH_DES3_2KEY, false);
        
    }
    
    public void setSessionKey(byte[] buffer, short offset) {
            // Check for empty keys
            if(Util.arrayCompare(buffer, (short)0, EMPTY_KEY, (short)0, KEY_SIZE) != 0) {
                  keyENC.setKey(buffer, (short) 0);
                  cipher.init(keyENC, Cipher.MODE_ENCRYPT);
                  decipher.init(keyENC, Cipher.MODE_DECRYPT);
            }
    }
    
    public static String bytesToHex(byte[] data) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < data.length; i++) {
            buf.append(byteToHex(data[i]));
            buf.append(" ");
        }
        return (buf.toString());
    }
    
    public short wrapResponseAPDU(short offset, short length) {
        byte[] buffer = APDU.getCurrentAPDUBuffer();
        
        // Encrypt whole body of the response.
        byte[] out = new byte[512];
        short outLen = cipher.doFinal(buffer, (short) offset, (short) length, out, (short) 0);
        // Write encrypted body to the response.
        Util.arrayCopyNonAtomic(out, (short) 0, buffer, (short) offset, outLen);
        return outLen;
    }
        
    public void unwrapCommandAPDU() {
        byte[] buffer = APDU.getCurrentAPDUBuffer();
        short apdu_p = (short) (ISO7816.OFFSET_CDATA & 0xff);
        short length = (short) buffer[ISO7816.OFFSET_LC & 0xff];
        
        // Decrypt only the body of the APDU.
        byte[] out = new byte[512];
        short outLen = decipher.doFinal(buffer, (short) apdu_p, (short) length, out, (short) 0);
        
        // Write decrypted body to the APDU.
        Util.arrayCopyNonAtomic(out, (short) 0, buffer, (short) apdu_p, outLen);
        buffer[ISO7816.OFFSET_LC & 0xff] = (byte) outLen;
    }
    
    /**
     * Desktop side methods.
     * @param response
     * @return decrypted response
     */
    
    public byte[] decryptResponse(ResponseAPDU response){
        byte[] buffer = response.getBytes();
        short bufLen = (short) buffer.length;
        short statusLen = (short)2;
        
        if (bufLen <= statusLen) {
            return buffer;
        }
        
        // Do not decrypt the last two bytes - they are status bytes.
        int length = bufLen - statusLen;
        byte[] out = new byte[256];
        short outLen = decipher.doFinal(buffer, (short) 0, (short) length, out, (short) 0);
        
        // Write decrypted response.
        byte[] apdu = new byte[outLen + statusLen];
        Util.arrayCopyNonAtomic(out, (short)0, apdu, (short)0, outLen);
        Util.arrayCopyNonAtomic(buffer, (short)length, apdu, (short)outLen, statusLen);
        return apdu;
    }
    
    /**
     *  Encrypt the full APDU.
     */
     
    public byte[] encryptSend(byte[] buffer){
        short apdu_p = (short) (ISO7816.OFFSET_CDATA & 0xff);
        short length = (short) buffer[ISO7816.OFFSET_LC & 0xff];
        
        // Encrypt only the body of the APDU.
        byte[] out = new byte[256];
        short outLen = cipher.doFinal(buffer, (short) apdu_p, (short) length, out, (short) 0);
        
        // Write recreate the APDU with encrypted body.
        byte[] apdu = new byte[apdu_p + outLen];
        Util.arrayCopyNonAtomic(buffer, (short)0, apdu, (short)0, apdu_p);
        Util.arrayCopyNonAtomic(out, (short)0, apdu, apdu_p, outLen);
        apdu[ISO7816.OFFSET_LC & 0xff] = (byte) outLen;
        return apdu;
    }
}