/*
 * JPass
 *
 * Copyright (c) 2009-2017 Gabor Bata
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package jpass.ui.helper;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import jpass.data.DocumentHelper;
import jpass.smartcard.CardsManager;
import jpass.ui.JPassFrame;
import jpass.ui.MessageDialog;
import jpass.ui.action.Callback;
import jpass.ui.action.Worker;
import jpass.util.StringUtils;

/**
 * Helper utils for file operations.
 *
 * @author Gabor_Bata
 *
 */
public final class FileHelper {
    
    private FileHelper() {
        // not intended to be instantiated
    }

    /**
     * Creates a new entries document.
     *
     * @param parent parent component
     */
    public static void createNew(final JPassFrame parent) {
        if (parent.getModel().isModified()) {
            int option = MessageDialog.showQuestionMessage(
                    parent,
                    "The current file has been modified.\n"
                    + "Do you want to save the changes before closing?",
                    MessageDialog.YES_NO_CANCEL_OPTION);
            if (option == MessageDialog.YES_OPTION) {
                saveFileCard(parent, false, new Callback() {
                    @Override
                    public void call(boolean result) {
                        if (result) {
                            parent.getModel().clear();
                            parent.getSearchPanel().setVisible(false);
                            parent.refreshAll();
                        }
                    }
                });
                return;
            } else if (option != MessageDialog.NO_OPTION) {
                return;
            }
        }
        parent.getModel().clear();
        parent.getSearchPanel().setVisible(false);
        parent.refreshAll();
    }

    /**
     * Shows a filechooser dialog and saves a file.
     *
     * @param parent parent component
     * @param saveAs normal 'Save' dialog or 'Save as'
     */
    public static void saveFileCard(final JPassFrame parent, final boolean saveAs) {
        saveFileCard(parent, saveAs, new Callback() {
            @Override
            public void call(boolean result) {
                //default empty call
            }
        });
    }

    /**
     * Shows a filechooser dialog and saves a file.
     *
     * @param parent parent component
     * @param saveAs normal 'Save' dialog or 'Save as'
     * @param callback callback function with the result; the result is
     * {@code true} if the file successfully saved; otherwise {@code false}
     */
    public static void saveFileCard(final JPassFrame parent, final boolean saveAs, final Callback callback) {
        CardsManager instance = CardsManager.getInstance();
        byte[] pin = null;
        
        if (instance.initalizeCard()) {
            System.out.println("CARD CONNECTED");
            
            boolean verified = instance.isPinUnlocked();
            
            while (!verified) {
                pin = MessageDialog.showPINDialog(parent);
                if (pin == null) {
                    return;
                }
                verified = instance.verifyPin(pin);
            }
            
            final String fileName;
            if (saveAs || parent.getModel().getFileName() == null) {
                File file = showFileChooser(parent, "Save", "jpass", "JPass Data Files (*.jpass)");
                if (file == null) {
                    callback.call(false);
                    return;
                }
                fileName = checkExtension(file.getPath(), "jpass");
                if (!checkFileOverwrite(fileName, parent)) {
                    callback.call(false);
                    return;
                }
            } else {
                fileName = parent.getModel().getFileName();
            }
            
            Worker worker = new Worker(parent) {
                @Override
                protected Void doInBackground() throws Exception {
                    try {
                        DocumentHelper.newInstance(fileName).writeDocumentCard(parent.getModel().getEntries());
                        parent.getModel().setFileName(fileName);
                        parent.getModel().setPassword(null);
                        parent.getModel().setModified(false);
                    } catch (Throwable e) {
                        throw new Exception("An error occured during the save operation:\n" + e.getMessage());
                    }
                    return null;
                }
                
                @Override
                protected void done() {
                    stopProcessing();
                    boolean result = true;
                    try {
                        get();
                    } catch (Exception e) {
                        result = false;
                        showErrorMessage(e);
                    }
                    callback.call(result);
                }
            };
            worker.execute();
        } else {
            System.out.println("CARD NOT CONNECTED");
            MessageDialog.showWarningMessage(parent, "Please insert your JavaCard\n");
            return;
        }
    }

    /**
     * Shows a filechooser dialog and opens a file.
     *
     * @param parent parent component
     */
    public static void openFileCard(final JPassFrame parent) {
        try {
            CardsManager instance = CardsManager.getInstance();
            byte[] pin = null;
            
            if (instance.initalizeCard()) {
                boolean verified = instance.isPinUnlocked();
                
                while (!verified) {
                    pin = MessageDialog.showPINDialog(parent);
                    if (pin == null) {
                        return;
                    }
                    verified = instance.verifyPin(pin);
                }
                
                final File file = showFileChooser(parent, "Open", "jpass", "JPass Data Files (*.jpass)");
                if (file == null) {
                    return;
                }
                if (parent.getModel().isModified()) {
                    int option = MessageDialog.showQuestionMessage(
                            parent,
                            "The current file has been modified.\n"
                            + "Do you want to save the changes before closing?",
                            MessageDialog.YES_NO_CANCEL_OPTION);
                    if (option == MessageDialog.YES_OPTION) {
                        saveFileCard(parent, false, new Callback() {
                            @Override
                            public void call(boolean result) {
                                if (result) {
                                    doOpenFileCard(file.getPath(), parent);
                                }
                            }
                        });
                        return;
                    } else if (option != MessageDialog.NO_OPTION) {
                        return;
                    }
                }
                doOpenFileCard(file.getPath(), parent);
                
            } else {
                System.out.println("CARD NOT CONNECTED");
                MessageDialog.showWarningMessage(parent, "Please insert your JavaCard\n");
                return;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Loads a file and fills the data model.
     *
     * @param fileName file name
     * @param parent parent component
     */
    public static void doOpenFileCard(final String fileName, final JPassFrame parent) {
        parent.getModel().clear();
        if (fileName == null) {
            return;
        }
        
        Worker worker = new Worker(parent) {
            @Override
            protected Void doInBackground() throws Exception {
                try {
                    parent.getModel().setEntries(DocumentHelper.newInstance(fileName).readDocumentCard());
                    parent.getModel().setFileName(fileName);
                    parent.getModel().setPassword(null);
                    parent.getSearchPanel().setVisible(false);
                } catch (FileNotFoundException e) {
                    throw e;
                } catch (IOException e) {
                    throw new Exception("An error occured during the open operation.\nPlease check your password.");
                } catch (Throwable e) {
                    throw new Exception("An error occured during the open operation:\n" + e.getMessage());
                }
                return null;
            }
            
            @Override
            protected void done() {
                stopProcessing();
                try {
                    get();
                } catch (Exception e) {
                    if (e.getCause() != null && e.getCause() instanceof FileNotFoundException) {
                        handleFileNotFound(parent, fileName, null);
                    } else {
                        showErrorMessage(e);
                    }
                }
            }
        };
        worker.execute();
    }
     

    /**
     * Handles file not found exception.
     *
     * @param parent parent frame
     * @param fileName file name
     * @param password password to create a new file
     */
    static void handleFileNotFound(final JPassFrame parent, final String fileName, final byte[] password) {
        int option = MessageDialog.showQuestionMessage(parent, "File not found:\n" + StringUtils.stripString(fileName)
                + "\n\nDo you want to create the file?", MessageDialog.YES_NO_OPTION);
        if (option == MessageDialog.YES_OPTION) {
            Worker fileNotFoundWorker = new Worker(parent) {
                @Override
                protected Void doInBackground() throws Exception {
                    try {
                        DocumentHelper.newInstance(fileName, password).writeDocumentCard(parent.getModel().getEntries());
                        parent.getModel().setFileName(fileName);
                        parent.getModel().setPassword(null);
                    } catch (Exception ex) {
                        throw new Exception("An error occured during the open operation:\n" + ex.getMessage());
                    }
                    return null;
                }
                
            };
            fileNotFoundWorker.execute();
        }
    }

    /**
     * Shows a filechooser dialog.
     *
     * @param parent parent component
     * @param taskName name of the task
     * @param extension accepted file extension
     * @param description file extension description
     * @return a file object
     */
    private static File showFileChooser(final JPassFrame parent, final String taskName,
            final String extension, final String description) {
        File ret = null;
        JFileChooser fc = new JFileChooser("./");
        fc.setFileFilter(new FileFilter() {
            @Override
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith("." + extension);
            }
            
            @Override
            public String getDescription() {
                return description;
            }
        });
        int returnVal = fc.showDialog(parent, taskName);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            ret = fc.getSelectedFile();
        }
        return ret;
    }

    /**
     * Checks if overwrite is accepted.
     *
     * @param fileName file name
     * @param parent parent component
     * @return {@code true} if overwrite is accepted; otherwise {@code false}
     */
    private static boolean checkFileOverwrite(String fileName, JPassFrame parent) {
        boolean overwriteAccepted = true;
        File file = new File(fileName);
        if (file.exists()) {
            int option = MessageDialog.showQuestionMessage(parent, "File is already exists:\n" + StringUtils.stripString(fileName)
                    + "\n\nDo you want to overwrite?", MessageDialog.YES_NO_OPTION);
            if (option != MessageDialog.YES_OPTION) {
                overwriteAccepted = false;
            }
        }
        return overwriteAccepted;
    }

    /**
     * Checks if the file name has the given extension
     *
     * @param fileName file name
     * @param extension extension
     * @return file name ending with the given extenson
     */
    private static String checkExtension(final String fileName, final String extension) {
        String separator = fileName.endsWith(".") ? "" : ".";
        if (!fileName.toLowerCase().endsWith(separator + extension)) {
            return fileName + separator + extension;
        }
        return fileName;
    }
}
